@@ -4,7 +4,7 @@
 #' and, optionally, standard error for each regressor.
 #'
 #' @param y numeric vector of observations to model
-#' @param ... numeric vectors of variables used in the modeling
+#' @param ... numeric vectors of variables used in the modelling
 #' @param se logical indicating whether to compute the standard error
 #'
 #' @return
@@ -14,6 +14,8 @@
 #'    \item{estimate}{estimate of the regression}
 #'    \item{std.error}{standard error}
 #'    \item{df}{degrees of freedom}
+#'    \item{r.squared}{Percent of variance explained by the model (repeated in each term)}
+#'    \item{adj.r.squared}{ r.squared` adjusted based on the degrees of freedom)}
 #' }
 #'
 #' If there's no complete cases in the regression, `NA`s are returned with no
@@ -27,14 +29,14 @@
 #'   regr <- geopotential[, FitLm(gh, date, se = TRUE), by = .(lon, lat)]
 #' })
 #'
-#' ggplot(regr[term != "(intercept)"], aes(lon, lat)) +
+#' ggplot(regr[term != "(Intercept)"], aes(lon, lat)) +
 #'     geom_contour(aes(z = estimate, color = ..level..)) +
 #'     stat_subset(aes(subset = abs(estimate) > 2*std.error), size = 0.05)
 #'
 #' # Using stats::lm() is much slower and with no names.
 #' \dontrun{
 #' system.time({
-#'   regr <- geopotential[, coef(lm(gh ~ date)), by = .(lon, lat)]
+#'   regr <- geopotential[, coef(lm(gh ~ date))[2], by = .(lon, lat)]
 #' })
 #' }
 #'
@@ -43,6 +45,9 @@
 FitLm <- function(y, ..., se = FALSE) {
     X <- cbind(`(Intercept)` = 1, ...)
     term <- dimnames(X)[[2]]
+    missing <- term == ""
+    term[missing] <- paste0("V", seq_len(sum(missing)))
+
     remove <- which(!complete.cases(X) | is.na(y))
     N <- length(y) - length(remove)
 
@@ -55,7 +60,9 @@ FitLm <- function(y, ..., se = FALSE) {
             return(list(term = term,
                         estimate = estimate,
                         std.error = se,
-                        df = rep(df, length(term))))
+                        df = rep(df, length(term)),
+                        r.quared = rep(NA_real_, length(term)),
+                        adj.r.squared = rep(NA_real_, length(term))))
         } else {
             return(list(term = term,
                         estimate = estimate))
@@ -72,16 +79,23 @@ FitLm <- function(y, ..., se = FALSE) {
 
     if (se == TRUE) {
         df <- N - ncol(X)
+        res_sum <- sum(a$residuals^2)
+        ss <- sum((y - mean(y))^2)
+        r_squared <- 1 - res_sum/ss
+        adj_r_squared <- 1 - res_sum/ss*(N-1)/df
+
         if (all(a$residuals == 0)) {
             se <- NA_real_
         } else {
-            sigma <- sum(a$residuals^2)/(nrow(X) - ncol(X))
+            sigma <- res_sum/(nrow(X) - ncol(X))
             se <- sqrt(diag(chol2inv(chol(t(X)%*%X)))*sigma)
         }
         return(list(term = term,
                     estimate = estimate,
                     std.error = se,
-                    df = rep(df, length(term))))
+                    df = rep(df, length(term)),
+                    r.squared = rep(r_squared, length(term)),
+                    adj.r.squared = rep(adj_r_squared, length(term))))
     } else {
         return(list(term = term,
                     estimate = estimate))
